"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var uuidv4 = require("uuid/v4");
module.exports = /** @class */ (function () {
    function ExpressData(config) {
        this.verbose = config ? config.verbose || false : false;
        this.onEndRequest = config ? config.onEndRequest || false : false;
    }
    ExpressData.prototype.timeElapsed = function (startTime, endTime) {
        if (!startTime || !endTime) {
            return 0;
        }
        return ((endTime[0] - startTime[0]) * 1e3 + (endTime[1] - startTime[1]) * 1e-6);
    };
    ExpressData.prototype.before = function (object, method, fn) {
        var originalMethod = object[method];
        object[method] = function () {
            fn.apply(object);
            originalMethod.apply(object, arguments);
        };
    };
    ExpressData.prototype.after = function (object, method, fn) {
        var originalMethod = object[method];
        object[method] = function () {
            originalMethod.apply(object, arguments);
            fn.call(object);
        };
    };
    ExpressData.prototype.interceptor = function (req, res, next) {
        var _this = this;
        var requestOpen = true;
        var events = {
            uuid: uuidv4(),
            url: req.url,
            method: req.method,
            startTime: process.hrtime()
        };
        var warnNull = function (name) { return console.warn("An event with name " + name + " has NOT been created. Use the record method to created it."); };
        var warnExists = function (name) { return console.warn("An event with name " + name + " has already been created."); };
        var init = function (name) {
            if (!events[name]) {
                events[name] = {};
            }
            else {
                warnExists(name);
            }
        };
        var record = function (name) {
            init(name);
            events[name].startTime = process.hrtime();
        };
        var stop = function (name) {
            if (!events[name]) {
                warnNull(name);
            }
            else {
                events[name].stopTime = process.hrtime();
                events[name].timeElapsed = _this.timeElapsed(events[name].startTime, events[name].stopTime);
            }
            if (_this.verbose) {
                logEvent(name);
            }
        };
        var endRequest = function (cb) {
            if (requestOpen) {
                events.stopTime = process.hrtime();
                events.timeElapsed = _this.timeElapsed(events.startTime, events.stopTime);
                _this.onEndRequest ? _this.onEndRequest(req) : null;
                cb ? cb() : null;
            }
            requestOpen = false;
        };
        var logEvent = function (name) { return console.info(name + ": (" + events[name].timeElapsed + "ms)"); };
        var getData = function () { return events; };
        this.after(res, 'send', endRequest);
        this.after(res, 'sendFile', endRequest);
        this.after(res, 'render', endRequest);
        this.after(res, 'sendStatus', endRequest);
        this.after(res, 'json', endRequest);
        this.after(res, 'jsonp', endRequest);
        this.after(res, 'end', endRequest);
        req.expressData = {
            record: record,
            stop: stop,
            endRequest: endRequest,
            logEvent: logEvent,
            getData: getData
        };
        next();
    };
    return ExpressData;
}());
